/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Preferences Toolbar 3.
 *
 * The Initial Developer of the Original Code is Manuel Reimer.
 *
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Aaron Andersen <aaron@xulplanet.com>
 *                 Kevin Teuscher
 *                 Manuel Reimer <Manuel.Reimer@gmx.de>
 *                 Stephen Clavering <mozilla@clav.me.uk>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var prefbarContextShiftDown = false;

//var prefbarFocusEnabled = false;

var prefbarDatabaseChanged = false;

var chevron_enabled = true;

window.addEventListener("load", prefbarStartPrefBar, true);

function prefbarStartPrefBar(event) {
    window.removeEventListener("load", prefbarStartPrefBar, true);

    var buttons = document.getElementById("prefbar-buttons");
    var chevron = document.getElementById("prefbar-chevron-popup");

    goPrefBar.RDF.AddmDatasource(buttons);
    goPrefBar.RDF.AddmDatasource(chevron);

    var menubar = document.getElementById("main-menubar");
    if (menubar) {
        var prefbarmenu = document.createElement('menu');
	var prefbarpopup = document.createElement('menupopup');
	prefbarmenu.setAttribute("id", "prefbar-menu");
	prefbarmenu.setAttribute("label", "PrefBar");
	prefbarpopup.setAttribute("datasources", "rdf:null");
	prefbarpopup.setAttribute("template", "prefbarToolbarTemplate");
	prefbarpopup.setAttribute("ref", "urn:prefbar:buttons");
	prefbarpopup.setAttribute("flags", "dont-test-empty");
	prefbarpopup.setAttribute("id", "prefbar-menu-popup");
	prefbarpopup.setAttribute("onpopuphidden", "prefbarSetChecks(false, false);");
	prefbarpopup.setAttribute("onpopupshowing", "prefbarSetChecks(true, false, this);");
        menubar.appendChild(prefbarmenu);
	prefbarmenu.appendChild(prefbarpopup);
	goPrefBar.RDF.AddmDatasource(prefbarpopup);
        // Set "collapsed" attribute. This, for any reason, has to be set *after* all
        // Elements are linked together.
        prefbarmenu.setAttribute("collapsed", Boolean(!navigator.preference("extensions.prefbar.show_prefbar_menu")));
    }

    buttons.database.AddObserver(prefbarRDFObserver);

    // Register a few event listeners
    window.addEventListener("focus", prefbarFocus, false);
    window.addEventListener("resize", prefbarResize, false);
    //window.addEventListener("close", prefbarClosePrefBar, true);
    
    // Bugfix for Firefox. An old workaround to toggle the toolbar forces us to add the following lines
    if (goPrefBar.InFirefox) {
	var toolbar = document.getElementById("prefbar");
	toolbar.setAttribute("collapsed", false);
    }
    
    // Move PrefBar if needed
    prefbarDisplayOn(navigator.preference("extensions.prefbar.display_on"));
}


// preferences observer "driven" by goPrefBar
function prefbarPrefObserver(subject, topic, data) {
    if(data == "extensions.prefbar.show_prefbar_menu") {
        var value = navigator.preference("extensions.prefbar.show_prefbar_menu");
        document.getElementById("prefbar-menu").setAttribute("collapsed", !value);
    }
    if (data == "extensions.prefbar.display_on") {
        prefbarDisplayOn(navigator.preference("extensions.prefbar.display_on"));
    }
}


function prefbarDisplayOn (targetindex) {
    var key = document.getElementById("key_prefbar");
    var menuitem = document.getElementById("viewprefsbar");
    var buttons = document.getElementById("prefbar-buttons");
    var contents = document.getElementById("prefbar-contents");
    var toolbars = [document.getElementById("prefbar"),
		    document.getElementById("main-menubar"),
		    document.getElementById("nav-bar"),
		    document.getElementById("PersonalToolbar")];
    
    if (targetindex == 0) {
	key.command = "cmd_prefbar";
	menuitem.collapsed = false;
	toolbars[0].collapsed = false;
    }
    else {
	key.command = "";
	menuitem.collapsed = true;
	toolbars[0].collapsed = true;
    }

    if (targetindex == 2 || targetindex == 3) {
	contents.flex = "0";
	chevron_enabled = false;
    }
    else {
	contents.flex = "1";
	chevron_enabled = true;
    }

    var currentParent = contents.parentNode;
    if (currentParent != toolbars[targetindex]) {
	currentParent.removeChild(contents);
	toolbars[targetindex].appendChild(contents);
	buttons.builder.rebuild();
	prefbarUpdateToolbar();
	if ("BookmarksToolbar" in this && "resizeFunc" in BookmarksToolbar)
	    BookmarksToolbar.resizeFunc();
    }
}

    
function prefbarResize(event) {
    if (event.target != document && event.target != document.getElementById("prefbar-chevron-popup")) {
	//	goPrefBar.dump("prefbarResize: ignored: " + event.target + " " + event.target.getAttribute("id"));
	return;
    }

    // Check if we are in an popup window and if the user has configured us not
    // to display there. For any reason the "chromehidden"-Attribute doesn't
    // exist in the "load"-Event and has to be catched here.
    if (!navigator.preference("extensions.prefbar.show_in_popups")) {
	if (document.getElementById("main-window").getAttribute("chromehidden").match(/toolbar/)) {
	    // Collapse Toolbar
	    document.getElementById("prefbar").setAttribute("collapsed", true);
	    // Disable the Shortcut "F8" for this window
	    document.getElementById("key_prefbar").command = "";
	    return;
	}
    }

    goPrefBar.dump("Resize");
    prefbarUpdateToolbar();
    //prefbarFocusEnabled = true;
}


function prefbarFocus(event) {
    if (event.target != document) return;
    // Silly workaround to get the first "focus" event after the first "resize"
    // event as most buttons on toolbar are collapsed then (less CPU usage).
    //if (!prefbarFocusEnabled) {
    //  goPrefBar.dump("BlockedFocus");
    //  return;
    //}

    goPrefBar.dump("Focus");

    if (prefbarDatabaseChanged) {
	prefbarUpdateToolbar();
	prefbarDatabaseChanged = false;
    }
    else
	prefbarSetChecks(false, false);
}


function prefbarUpdateToolbar() {
    // Force to redraw everything as we need the button width.
    prefbarSetChecks(false, true);

    goPrefBar.dump("UpdateToolbar");
   
    var buttons = document.getElementById("prefbar-buttons");
//    if (!buttons)
//      return;

    var chevron = document.getElementById("prefbar-chevron");
    if (!buttons.firstChild) {
	// No buttons means no chevron
	chevron.collapsed = true;
	return;
    }

    var spacers = new Array()

    chevron.collapsed = false;
    var chevronWidth = chevron.boxObject.width;
    chevron.collapsed = true;

    var toolbarWidth = window.innerWidth; // buttons.boxObject.width;
    var overflowed = false;

    for (var i=0; i<buttons.childNodes.length; i++) {
	var button = buttons.childNodes[i];
	if (chevron_enabled) {
	    button.collapsed = overflowed;
	      
	    if (i == buttons.childNodes.length - 1)
		chevronWidth = 0;

	    if (button.tagName == "spacer") {
		spacers.push(button);
		button.setAttribute("hidden", true);
		continue;
	    }

	    var remainingWidth = toolbarWidth - (button.boxObject.x + button.boxObject.width);
	    if (remainingWidth < chevronWidth) {
		overflowed = true;
		// This button doesn't fit. Show it in the menu. Hide it in the toolbar.
		if (!button.collapsed)
		    button.collapsed = true;
		if (chevron.collapsed) {
		    chevron.collapsed = false;
		}
	    }
	}
	else {
	    button.collapsed = false;
	    chevron.collapsed = true;
	}
    }

    for (var s=0; s<spacers.length; s++) {
	spacers[s].setAttribute("hidden", false);
    }
}


function prefbarUpdateMenu() {   
    goPrefBar.dump("UpdateMenu");
    
    prefbarSetChecks(true, false);

    var toolbar = document.getElementById("prefbar-buttons");
    var menu = document.getElementById("prefbar-chevron-popup");
    var spacercount = 0;

    for (var i = 0; i < toolbar.childNodes.length; i++) {
      var button = toolbar.childNodes[i];
      if (button.tagName == "spacer") {
	  spacercount++;
	  continue;
      }
      var menuitem = menu.childNodes[i - spacercount];
      if (menuitem.hidden == button.collapsed)
	  menuitem.hidden = !menuitem.hidden;
    }
}


function prefbarSetChecks(menu, forceAll, optional_target) {
    goPrefBar.dump("SetChecks Menu:" + !!menu);

    var buttons;

    if (optional_target != undefined)
	buttons = optional_target;
    else {
	if (!menu)
	    buttons = document.getElementById("prefbar-buttons");
	else
	    buttons = document.getElementById("prefbar-chevron-popup");
    }

    for (var i=0; i<buttons.childNodes.length; i++) {


	var button = buttons.childNodes[i];
	if (!forceAll && button.collapsed == true) {
	    goPrefBar.dump("SetChecks: jumpingbutton");
	    break;
	}
	var btntype = button.getAttribute('btntype');

	switch (btntype) {
	case "check":
	    prefbarSetCheck(button, menu);
	    break;
	case "extcheck":
	    prefbarSetExtcheck(button, menu);
	    break;
	case "menulist":
	    if (!menu) prefbarSetMenulist(button.menupopup, false);
	    break;
	case "extlist":
	    if (!menu) prefbarSetExtlist(button.menupopup, false);
	    break;
	}
    }
}

function prefbarSetCheck(button, menu) {
    var value = navigator.preference(button.getAttribute("prefstring")); // Value is magic variable referenced in prefstring

    //goPrefBar.dump("prefbarSetCheck: itemId: " + button.getAttribute("id"));

    var checked = eval(button.getAttribute("frompref"));

    if (!menu)
	button.setAttribute("checked", checked);
    else {
	var tvalue = checked ? "checkbox" : "radio";
	button.setAttribute("type", tvalue);
	button.setAttribute("checked", "true");
    }    
}

function prefbarProcessCheck(button, menu) {
    var value;
    if (!menu)
	value = button.checked;
    else
	value = (button.getAttribute("type") == "radio");

    navigator.preference(button.getAttribute("prefstring"), eval(button.getAttribute("topref")));
}

function prefbarSetExtcheck(button, menu) {
    var func = button.getAttribute("getfunction");

    var value;
    eval(func);

    if (!menu)
	button.setAttribute("checked", value);
    else {
	var tvalue = value ? "checkbox" : "radio";
	//goPrefBar.dump(tvalue);
	button.setAttribute("type", tvalue);
	button.setAttribute("checked", "true");
    }   
}

function prefbarProcessExtcheck(button, menu) {
    var func = button.getAttribute("setfunction");

    var value;
    if (!menu)
	value = button.checked;
    else
	value = (button.getAttribute("type") == "radio");

    eval(func);

    //prefbarSetExtcheck(item.getAttribute("id"));
}

function prefbarSetMenulist(menupopup, menu) {
    var prefstring = menupopup.getAttribute("prefstring");

    var nsIPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var prefBranch = nsIPref.getBranch("");
    var prefHasUserValue = prefBranch.prefHasUserValue(prefstring);

    var prefvalue;
    try {
	prefvalue = String(navigator.preference(prefstring));
    }
    catch(e) {
	prefvalue = undefined;
    }

    var activeitem = prefbarGenerateMenupopup(menupopup, menu, !prefHasUserValue, prefvalue, "prefbarProcessMenulist(this)");
    if (!menu)
	menupopup.parentNode.selectedItem = activeitem;
    else
	activeitem.setAttribute("checked", "true");
}

function prefbarProcessMenulist(menuitem) {
    var nsIPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var prefBranch = nsIPref.getBranch("");

    var pref = menuitem.parentNode.getAttribute("prefstring");
    var value = menuitem.getAttribute("value");
    var type = prefBranch.getPrefType(pref);

    if (value == "PREFBARDEFAULT")
	prefBranch.clearUserPref(pref);
    // XXX Hack to get backwards compatibility
    // In new PrefBar versions the default item in an list is marked by the
    // value "PREFBARDEFAULT". This allows to have the default item everywhere
    // in the list and doesn't confuse the users (see PrefBar-Bug 5733) In old
    // versions simply the first item was the default. As we don't update the
    // items of the "User Agent Menulist", we have to be backwards compatible
    // for this one item.
    else if (value == "" && menuitem.parentNode.getAttribute("id") == "urn:prefbar:buttons:useragent")
	prefBranch.clearUserPref(pref);
    else {
	if(type == prefBranch.PREF_STRING)
	    prefBranch.setCharPref(pref, value);
	else if(type == prefBranch.PREF_INT)
	    prefBranch. setIntPref(pref, Number(value));
	else if(type == prefBranch.PREF_BOOL)
	    prefBranch.setBoolPref(pref, Boolean(value));
	else
	    navigator.preference(pref, value);  // Couldn't hurt to try...
    }
}


function prefbarSetExtlist(menupopup, menu) {
    var getfunction = menupopup.getAttribute("getfunction");

    var value = 0;
    eval(getfunction);
  
    var activeitem = prefbarGenerateMenupopup(menupopup, menu, false, value, "prefbarProcessExtlist(this)");

    if (!menu)
	menupopup.parentNode.selectedItem = activeitem;
    else
	activeitem.setAttribute("checked", "true");
}


function prefbarProcessExtlist(menuitem) {
    var func = menuitem.parentNode.getAttribute("setfunction");
    var value = menuitem.getAttribute("value");
    eval(func);
}


function prefbarGenerateMenupopup(menupopup, menu, defaultset, value, cmdhandler) {
    var RDF = goPrefBar.RDF.RDFService;
    var Datasource = goPrefBar.RDF.mDatasource;

    var itemId = menupopup.getAttribute("id");
    goPrefBar.dump("prefbarGenerateMenupopup: itemId: " + itemId);
    var RitemId = RDF.GetResource(itemId);

    var activeitem = null;
    var defaultitem = null;

    var itemindex = 1;

    while(menupopup.firstChild) menupopup.removeChild(menupopup.firstChild);

    while(true) {
	var Roptlabel = RDF.GetResource("http://www.xulplanet.com/rdf/prefbar#optionlabel" + itemindex);
	var Noptlabel = Datasource.GetTarget(RitemId, Roptlabel, true);
	if (!Noptlabel) break;
	var optlabel = Noptlabel.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
	var Roptvalue = RDF.GetResource("http://www.xulplanet.com/rdf/prefbar#optionvalue" + itemindex);
	var Noptvalue = Datasource.GetTarget(RitemId, Roptvalue, true);
	//if (!Noptvalue) break;
	var optvalue = Noptvalue.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;

	var newitem = document.createElement('menuitem');
	//if (defaultset && menupopup.getAttribute("default") == itemindex - 1) activeitem = newitem;
	if (optvalue == "PREFBARDEFAULT") defaultitem = newitem;
	if (optvalue == value) activeitem = newitem;

	newitem.setAttribute("label", optlabel);
	newitem.setAttribute("value", optvalue);
	newitem.setAttribute("oncommand", cmdhandler);
	if (menu) {
	    newitem.setAttribute("type", "checkbox");
	    newitem.setAttribute("checked", "false");
	}
	menupopup.appendChild(newitem);

	itemindex++;
    }

    if (defaultset && defaultitem) activeitem = defaultitem;

    // XXX Hack to get backwards compatibility
    // In new PrefBar versions the default item in an list is marked by the
    // value "PREFBARDEFAULT". This allows to have the default item everywhere
    // in the list and doesn't confuse the users (see PrefBar-Bug 5733) In old
    // versions simply the first item was the default. As we don't update the
    // items of the "User Agent Menulist", we have to be backwards compatible
    // for this one item.
    if (defaultset && !defaultitem && itemId == "urn:prefbar:buttons:useragent")
	activeitem = menupopup.firstChild;

    return activeitem;
}

function prefbarGoLink(url, event) {
    var pref_opentab = navigator.preference("browser.tabs.opentabfor.middleclick");
    var pref_openwindow = navigator.preference("middlemouse.openNewWindow");
    var pref_tabbackground = navigator.preference("browser.tabs.loadInBackground");
    var browser = window.document.getElementById("content");

    if (!event) {
	browser.loadURI(url);
	window.content.focus();
    }
    else {
	if (event.shiftKey) pref_tabbackground = !pref_tabbackground;

	var middleClick = true;
	if (event.type == "command" && !event.ctrlKey && !event.metaKey)
	    middleClick = false;

	if (!middleClick) {
	    browser.loadURI(url);
	    window.content.focus();
	}
	else {
	    if (pref_opentab) {
		var tab = browser.addTab(url);
		if (!pref_tabbackground)
		    browser.selectedTab = tab;
	    } else if (pref_openwindow)
		window.open(url);
	}
    }
}

function prefbarTogglePrefBar() {
    goToggleToolbar('prefbar','viewprefsbar');
    prefbarUpdateToolbar();
    window.content.focus();
    //prefbarSetChecks();
}

function prefbarOpenPrefs() {
    if(goPrefBar.InFirefox)
	openDialog('chrome://prefbar/content/prefbar-bird-pref.xul', 'PrefBar', 'chrome');
    else
	goPreferences('navigator.xul', 'chrome://prefbar/content/preferencesPanel.xul', 'prefbarPanel');
}

function prefbarOnContextMenu(event) 
{
    // Don't allow this uninstaller for Firefox!
    if (goPrefBar.InFirefox) {
	document.getElementById("prefbar-uninstall").setAttribute("hidden", "true");
	return;
    }

    // Only show the unistall item if the shift-key is down!
    //if (!event.shiftKey) // Once again an bug :-( We have to catch the status
    // of the shift-Key from the onmouseup and onkeyup events of the "toolbar"
    // item as it is always false here.
    if (!prefbarContextShiftDown)
	document.getElementById("prefbar-uninstall").setAttribute("hidden", "true");
    else
	document.getElementById("prefbar-uninstall").setAttribute("hidden", "false");
}

function prefbarUninstall()
{
    if (!confirm(goPrefBar.GetString("prefbarOverlayStrings", "uninstallconfirm"))) return;
	
    var unreg = new prefbarExUnregisterer (
					   'chrome://prefbar/content/contents.rdf',
					   'jar:%chromeFolder%prefbar.jar!/locale/en-US/prefbar/contents.rdf',
					   'jar:%chromeFolder%prefbar.jar!/locale/de-AT/prefbar/contents.rdf'
					   );

    unreg.unregister();

    var PrefbarPath = unreg.UChrome + 'prefbar.jar';
    if (!unreg.exists(PrefbarPath)) PrefbarPath = unreg.Chrome + 'prefbar.jar';
    alert(goPrefBar.GetString("prefbarOverlayStrings", "uninstalldeletefile").replace("$filename", PrefbarPath));
}

var prefbarRDFObserver = {
    onAssert: function (aDataSource, aSource, aProperty, aTarget)  {
	prefbarDatabaseChanged = true;
    },
    onUnassert: function (aDataSource, aSource, aProperty, aTarget){},
    onChange: function (aDataSource, aSource, aProperty, aOldTarget, aNewTarget) {
	prefbarDatabaseChanged = true;
    },
    onMove: function (aDataSource, aOldSource, aNewSource, aProperty, aTarget) {},
    onBeginUpdateBatch: function (aDataSource) {},
    onEndUpdateBatch:   function (aDataSource) {}
}
